(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: identity_model_triple.ml,v 1.1 2014-02-19 15:01:02 fclement Exp $ *)

open Gen_identity_model;;

(* Triple stuff. *)

let get_dimension () = Nat._3_;;

let triple x0 x1 x2 = (x0, x1, x2);;
let triple_map f (x0, x1, x2) = (f x0, f x1, f x2);;
let triple_of_color c = (Colors.r_of c, Colors.g_of c, Colors.b_of c);;

let triple_get i (x0, x1, x2) =
  match Nat.to_int i with
  | 0 -> x0
  | 1 -> x1
  | 2 -> x2
  | _ -> assert false
;;

let triple_init f = (f Nat._0_, f Nat._1_, f Nat._2_);;

let triple_to_array (x0, x1, x2) = [| x0; x1; x2; |];;
let triple_of_array = function
  | [| x0; x1; x2; |] -> (x0, x1, x2)
  | _ -> i_invalid_arg "triple_of_array" "length of array is not 3"
;;

(* Color image stuff. *)

let of_color c = triple_map float_of_int (triple_of_color c);;

let to_color t =
  let (r, g, b) = triple_map int_of_float t in
  Colors.color_of_rgb r g b
;;

let data_of_image img =
  let im = Umatrix.map of_color img in
  let dms = triple_init (fun cn -> Umatrix.map (triple_get cn) im) in
  Umatrix.of_array_array (Array.map Umatrix.to_array (triple_to_array dms))
;;

let image_of_fine_parameter (ni, nj) pm =
  let (rm, gm, bm) =
    triple_of_array
      (Array.map
         (Uarray_ng.matrix_of_array ni nj)
         (Umatrix.to_array_array pm)) in
  Umatrix.map to_color (Uarray_ng.matrix_map3 triple rm gm bm)
;;
