(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2013-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: unumber_command_option.ml,v 1.5 2014-02-25 17:43:39 fclement Exp $ *)

module Cmd_opt = Ucommand_option;;

open Unumber_command_option_spec;;
  
let print_opt_doc ppf = function
  | None -> ()
  | Some msg -> Format.fprintf ppf "@[%s@]@ " msg
;;

let make_doc arg_name full_name opt_doc skind name print_default =
  let ppf = Format.str_formatter in
  Format.fprintf ppf
    "@[<v 8><%s>@ \
    @[Set %s to <%s>.@]@ \
    %a\
    @[%s is %s.@]@ \
    @[Default %s is %t.@]\
    @]"
    arg_name
    full_name arg_name
    print_opt_doc opt_doc
    (String.capitalize name) skind
    name print_default;
  Format.flush_str_formatter ()
;;

module Integer_Make =
  functor (S : STRUCTURE with type t = int) ->
struct
  open S;;
  
  type t = S.t;;
  
  let print ppf = Format.fprintf ppf "%d";;
  
  let { Uaref.get = get; set = set; } = Uaref.make_aref_single name default;;
  
  let arg_name =
    match kind with
    | Regular | Range _ -> "int"
    | Nonnegative | Positive -> "nat"
  ;;
  let skind =
    match kind with
    | Regular -> "an integer"
    | Nonnegative -> "a natural number"
    | Positive -> "a positive integer"
    | Range (s1, s2) -> Printf.sprintf "an integer between %s and %s" s1 s2
  ;;
  let print_default ppf = print ppf (get ());;
  let doc = make_doc arg_name full_name opt_doc skind name print_default;;
  
  let option =
    {
     Cmd_opt.key = Ustring.long_key_of_string name;
     spec = Arg.Int set;
     doc = doc;
    }
  ;;
end
;;

module Float_Make =
  functor (S : STRUCTURE with type t = float) ->
struct
  open S;;
  
  type t = S.t;;
  
  let print ppf = Format.fprintf ppf "%g";;
  
  let { Uaref.get = get; set = set; } = Uaref.make_aref_single name default;;
  
  let arg_name = "float";;
  let skind =
    match kind with
    | Regular -> "a floating-point number"
    | Nonnegative -> "a nonnegative floating-point number"
    | Positive -> "a positive floating-point number"
    | Range (s1, s2) ->
        Printf.sprintf"a floating-point number between %s and %s" s1 s2
  ;;
  let print_default ppf = print ppf (get ());;
  let doc = make_doc arg_name full_name opt_doc skind name print_default;;
  
  let option =
    {
     Cmd_opt.key = Ustring.long_key_of_string name;
     spec = Arg.Float set;
     doc = doc;
    }
  ;;
end
;;
